#!/usr/bin/env python

# Ubuntu Testing Automation Harness
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import jenkins
import jinja2
import logging
import os

from distro_info import UbuntuDistroInfo

DEV_SERIES = UbuntuDistroInfo().devel()
TESTS = ["default",
         "security",
         "sdk",
         "smem",
         "eventstat"]


def _get_parser():
    """Create and return command line parser.

    :returns: command line parser
    :rtype: argparse.ArgumentParser

    """
    parser = argparse.ArgumentParser(
        description='Create/Update upgrade testing jenkins jobs.')
    parser.add_argument("-d", "--dryrun", action="store_true",
                        help="Dry run mode. Don't execute jenkins commands.")
    parser.add_argument("-u", "--username",
                        help="username to use when logging into Jenkins.")
    parser.add_argument("-p", "--password",
                        help="username to use when logging into Jenkins")
    parser.add_argument("-j", "--jenkins", default="http://10.98.0.1:8080/",
                        help="URL of jenkins instance to configure jobs in.")
    parser.add_argument("-n", "--name", default="maguro-01",
                        help=("Device name where the job should be executed "
                              "(default=%(default)s)"))
    parser.add_argument("-s", "--series", default=DEV_SERIES,
                        help=("series of Ubuntu to download "
                        "(default=%(default)s)"))
    return parser


def _get_jenkins(url, username, password):
    logging.info('Attempting to login to jenkins at %s', url)
    if username is not None:
        logging.info('...with authentication as user: %s', username)
        instance = jenkins.Jenkins(url, username=username, password=password)
    else:
        logging.info('...without authentication')
        instance = jenkins.Jenkins(url)

    return instance


def _get_environment():
    base = os.path.dirname(__file__)+"/templates"
    return jinja2.Environment(
        loader=jinja2.FileSystemLoader(base),
        undefined=jinja2.StrictUndefined,
        trim_blocks=True)


def _get_job_name(args, testname):
    if (testname in ("default", "security", "sdk")):
        fmt = "{series}-touch-{type}-smoke-{testname}"
        return fmt.format(series=args.series,
            type=args.name[:args.name.index("-")],
            testname=testname)
    else:
        fmt = "{testname}-{series}-touch-armhf-install-idle-{type}"
        return fmt.format(series=args.series,
            testname=testname,
            type=args.name[:args.name.index("-")])


def _configure_job(instance, env, args, testname):
    tmpl_name = "touch-%s.xml.jinja2" % testname
    tmpl = env.get_template(tmpl_name)
    cfg = tmpl.render(name=args.name,
                      series=args.series,
                      type=args.name[:args.name.index("-")],
                      testname=testname)
    jobname = _get_job_name(args, testname)
    if args.dryrun:
        _dryrun_func(jobname, cfg)
        return
    if instance.job_exists(jobname):
        logging.info('reconfiguring job %s', jobname)
        instance.reconfig_job(jobname, cfg)
    else:
        logging.info('creating job %s', jobname)
        instance.create_job(jobname, cfg)


def _dryrun_func(jobname, config):
    logging.debug(jobname)
    logging.debug(config)


def main():
    logging.basicConfig(level=logging.DEBUG)
    args = _get_parser().parse_args()

    jenkins_inst = _get_jenkins(args.jenkins, args.username, args.password)
    if args.dryrun:
        jenkins_inst.create_job = _dryrun_func
        jenkins_inst.reconfig_job = _dryrun_func

    env = _get_environment()

    for testname in TESTS:
        logging.debug("configuring job for %s" %testname)
        _configure_job(jenkins_inst, env, args, testname)

if __name__ == '__main__':
    main()
